# AUTO-GENERATED FILE - DO NOT EDIT
# This file was automatically generated by the XDK build tool.
# Any manual changes will be overwritten on the next generation.
"""
Auto-generated stream models for the X API.

This module provides Pydantic models for request and response data structures
for the stream endpoints of the X API. All models are generated
from the OpenAPI specification and provide type safety and validation.

Generated automatically - do not edit manually.
"""

from typing import Dict, List, Optional, Any, Union, Literal
from pydantic import BaseModel, Field, ConfigDict
from datetime import datetime


# Models for posts_firehose


class PostsFirehoseResponse(BaseModel):
    """Response model for posts_firehose"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for posts_firehose_ja


class PostsFirehoseJaResponse(BaseModel):
    """Response model for posts_firehose_ja"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for posts_sample


class PostsSampleResponse(BaseModel):
    """Response model for posts_sample"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for likes_sample10


class LikesSample10Response(BaseModel):
    """Response model for likes_sample10"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for labels_compliance


class LabelsComplianceResponse(BaseModel):
    """Response model for labels_compliance"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for posts


class PostsResponse(BaseModel):
    """Response model for posts"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for posts_compliance


class PostsComplianceResponse(BaseModel):
    """Response model for posts_compliance"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for likes_compliance


class LikesComplianceResponse(BaseModel):
    """Response model for likes_compliance"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for get_rules


class GetRulesResponse(BaseModel):
    """Response model for get_rules"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for update_rules


class UpdateRulesRequest(BaseModel):
    """Request model for update_rules"""

    model_config = ConfigDict(populate_by_name=True)


class UpdateRulesResponse(BaseModel):
    """Response model for update_rules"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for users_compliance


class UsersComplianceResponse(BaseModel):
    """Response model for users_compliance"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for posts_sample10


class PostsSample10Response(BaseModel):
    """Response model for posts_sample10"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for posts_firehose_pt


class PostsFirehosePtResponse(BaseModel):
    """Response model for posts_firehose_pt"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for posts_firehose_ko


class PostsFirehoseKoResponse(BaseModel):
    """Response model for posts_firehose_ko"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for get_rule_counts


class GetRuleCountsResponse(BaseModel):
    """Response model for get_rule_counts"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for likes_firehose


class LikesFirehoseResponse(BaseModel):
    """Response model for likes_firehose"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for posts_firehose_en


class PostsFirehoseEnResponse(BaseModel):
    """Response model for posts_firehose_en"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")
