# AUTO-GENERATED FILE - DO NOT EDIT
# This file was automatically generated by the XDK build tool.
# Any manual changes will be overwritten on the next generation.
"""
Auto-generated lists models for the X API.

This module provides Pydantic models for request and response data structures
for the lists endpoints of the X API. All models are generated
from the OpenAPI specification and provide type safety and validation.

Generated automatically - do not edit manually.
"""

from typing import Dict, List, Optional, Any, Union, Literal
from pydantic import BaseModel, Field, ConfigDict
from datetime import datetime


# Models for get_by_id


class GetByIdResponse(BaseModel):
    """Response model for get_by_id"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for update


class UpdateRequest(BaseModel):
    """Request model for update"""

    model_config = ConfigDict(populate_by_name=True)


class UpdateResponse(BaseModel):
    """Response model for update"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for delete


class DeleteResponse(BaseModel):
    """Response model for delete"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for remove_member_by_user_id


class RemoveMemberByUserIdResponse(BaseModel):
    """Response model for remove_member_by_user_id"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for get_posts


class GetPostsResponse(BaseModel):
    """Response model for get_posts"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for create


class CreateRequest(BaseModel):
    """Request model for create"""

    model_config = ConfigDict(populate_by_name=True)


class CreateResponse(BaseModel):
    """Response model for create"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for get_members


class GetMembersResponse(BaseModel):
    """Response model for get_members"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for add_member


class AddMemberRequest(BaseModel):
    """Request model for add_member"""

    model_config = ConfigDict(populate_by_name=True)


class AddMemberResponse(BaseModel):
    """Response model for add_member"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")


# Models for get_followers


class GetFollowersResponse(BaseModel):
    """Response model for get_followers"""

    model_config = ConfigDict(populate_by_name=True, extra="allow")
